<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel <bartel@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: ZoneCreateForm.php 3941 2005-12-13 23:20:27Z bartel $

/**
 *  PreferencesForm - This file contains the {@link PreferencesForm} class
 *
 *  @author $Author:$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/MaintainForm.php");

/**
 *  This class contains the main preferences form. "Manage User Information"
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class PreferencesForm extends MaintainForm
{

/*Vars*/
//{{{   vars

    /**
     *  This array is used to map form labels to DBDO vars and vice versa
     */
    var $map = array("Username"=>"username",
                    "Email"=>"email",
                    "Default_Zone"=>"preferred_zone",
                    "View_Limit"=>"list_view",
                    "View_Type"=>"host_view",
                    "Color_Template"=>"color_template",
                    "Auth_Type" => "type");

//}}}
/*Methods*/
//{{{   form_init_elements()

    /**
     *  This method initializes the form elements that are going to be used
     */
    function form_init_elements()
    {
        if (isset($_REQUEST['username']))
            $user = new User($_REQUEST['username']);
        else
            global $user;
        
        $username = new FEText(_("Username"), false);
        $username->set_readonly(true);
        $email = new FEEmail(_("Email"), false);
        $last_seen = new FEText(_("Last Seen"), false);
        $last_seen->set_readonly(true);
        $pref_zone = new FEListBox(_("Default Zone"), false);
        $listview = new FEListBox(_("View Limit"), false);
        $hostview = new FEListBox(_("View Type"), false);
        $color = new FEListBox(_("Color Template"), false);
        $color->set_list_data($this->get_skins());
        /* Drop down to select auth type */
        $auth_type = new FEListBox(_("Auth Type"), false);
        /* Format the array correctly [auth_name][auth_name] */
        $auths = valid_auth_types();
        $arr = array();
        foreach ($auths as $auth)
            $arr[$auth] = $auth;
        
        $auth_type->set_list_data($arr);
        
        
        /* Add "Skin" selection here */
        /* Make the "Default Zone List box */        
        $zoneids = $user->zones();
        $zones = array();
        foreach($zoneids as $zoneid)    {
            $zone = new Zone($zoneid);
            $zones[$zone->name] = $zoneid;
        } 
        $pref_zone->set_list_data($zones);
        


        /*Get data for View Limit */
        //Change this later
        $view = array("25"=>25, "50"=>50, "75"=>75, "100"=>100);
        $listview->set_list_data($view);
        

        /* Handle the Host View (View Type) box */
        $types = array("Basic"=>'basic', "Advanced"=>'advanced');
        $hostview->set_list_data($types);


        $this->add_elements($username, $email, $last_seen, $pref_zone, $listview, $hostview, $color);
        global $user;
        if ($user->is_admin())
            $this->add_element($auth_type);
    }

//}}}
//{{{   form_init_data()

    function form_init_data()
    {
        if (isset($_REQUEST['username']))
            $user = new User($_REQUEST['username']);
        else
            global $user;

        $array = $user->toArray();

        $init = $this->remap($array, FORM);

        foreach($init as $label=>$val)  {
            if ($this->contains_element($label))    {
                $this->set_element_value($label, $val);
            }
        }
        
        $this->set_element_value(_("Last Seen"), format_date($user->last_seen));  

        $this->set_element_value(_("Default Zone"), $user->preferred_zone);
        $this->set_element_value(_("View Type"), $user->host_view);
        $this->set_element_value(_("Color Template"), $user->color_template);
        
    }
//}}}
//{{{   form_backend_validation()

    /**
     *  This method takes care of validation before the information is processed by
     *  the action method.
     *
     *  @return Boolean     True if data is valid
     */
    function form_backend_validation()
    {
        $res = true;
    
        if (isset($_REQUEST['username']))
            $user = new User($_REQUEST['username']);
        else
            global $user;

        $user->setFrom($this->remap($this->get_all_element_values(), DBDO));
        $valid = $user->validate();

        //if validation wasn't successful
        if($valid !== true) {
            
            $errors = $this->remap($valid, FORM);
            
            //Loop through the rrors
            foreach($errors as $label=>$error) {
                if($error !== true) //If the field failed
                    $this->add_error($label, $error); //report the error
            }
            
            $res = false;
            
        }
        
        return $res;
    }

//}}}
//{{{   form_action()

    /**
     *  This method processes the data after it passes validation
     *
     *  @return Boolean     True if processing was successful
     */
    function form_action()
    {
        /* See if there is a 'username' request var. we are editting a different user */
        if (isset($_REQUEST['username'])){
            $user = new User($_REQUEST['username']);
            $user->query("BEGIN");
        }
        else{
            global $user;
            $user->query("BEGIN");
        }    
        $res = true;
    
        switch(strtolower($this->get_action())) {
        
        case 'update':
            
            $user->setAll();
            $user->setFrom($this->remap($this->get_all_element_values(), DBDO));
            
            if ($user->update() === false)  {
                $user->query("ROLLBACK");
                $this->add_error(_("Error"), _("Update query failed"));
            }

            $user->query("COMMIT");
            break;
        
        default:
            die_hard(_("Invalid form action"));
            break;
        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  If the form_action was successful then this method will fire. We probably just want to stay on this page though.
     *
     *  @return DIVTag  $div    A div that contains a success message
     */
    function form_success()
    {
        header("Location: ".$_SERVER['SCRIPT_NAME']."?action=show&type=local");
    }

//}}}
//{{{   form_buttons()
    /**
     *
     * Overload this to customize the buttons
     * 
     * @return TRTag     $tr    A table row with buttons
     */
    
    function form_buttons($class = null)
    {
        $tr = html_tr($class);
        $this->set_cancel_action($_SERVER["SCRIPT_NAME"]);
        $tr->add("&nbsp;", container($this->add_action(_("Update")), $this->add_cancel()));
        return $tr;

    }
//}}}
    //{{{ get_skins()
        /*
         *  Returns a list of available skins for use in a listbox
         *
         *
         *
         */
     function get_skins()
     {
        $ary = array();
        $skindir = opendir(SKINDIR);
        while ($file = readdir($skindir))  {
            if (is_dir(SKINDIR."/".$file) && file_exists(SKINDIR."/$file/style.css"))
                $ary[$file] = $file;
        }
         
        return $ary;
     }
    //}}}
//{{{ form_hidden()
    /*
     *
     */
    function form_hidden()
    {
        $h = new FEHidden('action', $_REQUEST['action']);
        $this->add_element($h);
        $div = html_div();
        $div->add($this->element_form('action'));
        if (isset($_REQUEST['username']))   {
            $u = new FEHidden('username', $_REQUEST['username']);
            $this->add_element($u);
            $div->add($this->element_form('action'), $this->element_form('username'));
        }
        return $div;
    }
//}}}
}


/** PasswordForm
 *  This class is responsible for changing the user's password.
 *
 *  @package    htdocs
 *  @subpackage forms
 *
 */

class PasswordForm extends MaintainForm
{
   
//{{{   form_init_elements()     
    function form_init_elements()
    {
        $current = new FEPassword(_("Current Password"), true);
        $newpassword = new FEPassword(_("New Password"), true);
        $confirm = new FEConfirmPassword(_("Confirm Password"), true);
        $confirm->password($newpassword);

        $this->add_elements($current, $newpassword, $confirm);
        
    }
//}}}
//{{{ form_init_data()
    function form_init_data()
    {
        /*Do nothing?*/
    }
//}}}
//{{{   form_backend_validation()
    /* Make sure the user is who they say the are. Check given current password
     * to the users' current.
     *
     * @return Boolean  true if passwords match, false otherwise
     */
    function form_backend_validation()
    {
        global $user;

        $result = false;        

        $vals = $this->get_all_element_values();
        
        /* Check given password to users password */
        $pass = sha1($vals[_("Current_Password")]);
        
        if (strcasecmp($pass, $user->getPassword())) {
            $this->add_error(_("Current Password"), _("Incorrect Password"));
            return false;
        }
        
        if(strlen($vals[_("New_Password")]) < 8){
            $this->add_error(_("New Password"), _("Password must be 8 characters
                                or longer"));
            return false; 
        }

        if(preg_match("/^[A-Za-z]+$|^[0-9]+$|".$vals[_("Current_Password")].   
                               "/", $vals[_("New_Password")])){
            
           $this->add_error(_("New Password"), _("Password cannot contain all
                    characters, numbers, or contain your previous password"));
      
           return false; 
      
        }

        return true;
    }
//}}}
//{{{ form_action()
    function form_action()
    {
        global $user;
        switch(strtolower($this->get_action())) {
        
        case _("submit"):
            /*We got a good password now lets change the DB (fingers crossed) */
            $vals = $this->get_all_element_values();
            $newpass = $vals[_("New_Password")];
            $user->setPassword($newpass);
            $up = $user->update();
            if (!$up)   {
                $this->add_error(_("ERROR"), _("Could not update database"));
            }
            return true;
            break;
        default:
            die_hard(_("Invalid form action"));
            break;
        }
    }
//}}}
//{{{   form_success()
    function form_success()
    {
        $div = html_div("psuccess");
        //$indlink = html_a("index.php", "["._("Return to Home")."]");
        //$back = html_a("preferences.php", "["._("User Preferences")."]");
        $div->add(container(_("Password Changed"), html_br(2)));
        //$div->add(container($indlink, $back));
        return $div;
        
    }
//}}}

}

?>
