<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: RangeCreateForm.php 4722 2006-08-10 21:09:35Z morgabra $

/**
 *  RangeCreateForm.php - This file contains the {@link RangeCreateForm} class
 *
 *  @author $Author$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/MaintainForm.php");

/**
 *  This class is the form that users will use to create IP Ranges
 *
 *  @package    htdocs 
 *  @subpackage forms
 */
class RangeCreateForm extends MaintainForm
{

/*Vars*/
//{{{   vars

    /**
     *  This array maps form labels to the DBDO vars and vice versa
     */
    var $map = array("Name"=>"name",
                    "Start_IP"=>"start",
                    "End_IP"=>"end",
                    "Enabled"=>"enabled",
                    "Range_Type"=>"type",
                    "Any_Dynamic_Host"=>"allow_all_hosts",
                    "Overfill_Multiplier"=>"overfill",
                    "Comment"=>"comment");
    var $java;
    
//}}}
/*Methods*/
//{{{   form_init_elements()

    /**
     *  This method is responisble for initializing all the elements of the form
     */
    function form_init_elements()
    {
        $name = new FEText(_("Name"), true);
        $start = new FEText(_("Start IP"), true);
        $end = new FEText(_("End IP"), true);
        $enable = new FEListBox(_("Enabled"));
        $type = new FERadioGroup(_("Range Type"), array("Static"=>"static", " Dynamic"=>"dynamic"), true);
        $any = new FEYesNoListBox(_("Any Dynamic Host"),true,null,null,1,0);
        $overfill = new FEText(_("Overfill Multiplier"), false,  2);
        $comment = new FETextArea(_("Comment"),false,5,60);
        

        $enable->set_list_data(array("Enabled"=>1,
                                    "Disabled"=>0));

        /**
         * This javascript snippet handles the "on-click" behavior of the
         * $type radiobuttons. This will disable the overfill multiplier
         * field if 'static' is selected as the range's type, and vise versa
         * for dynamic ranges.
         */
        $js = "var thisform = document.".$this->get_form_name().";"
            ."var ovfl = thisform.".$overfill->get_element_name().";"
            ."if (thisform."._("Range_Type")."[0].checked)"
            ."{ ovfl.disabled = true; ovfl.value = '1.00';"
            ."} else {"
            ."ovfl.disabled = false;}";

        /**
         * This javascript snipped handles the pages' behavior on page load.
         * This is required because if the form errors out, the page will 
         * reload without any instructions from the "onclick" part, and the
         * overfill_multiplier field will be re-enabled regardless of the 
         * range type. 
         */
        $this->java = "window.onload = run();"
            ."function run() {"
            ."disable(\""._("Range_Type")."_0\"); "
            ."disable(\""._("Range_Type")."_1\"); } "
            ."function disable(ob) { "
            ."if(document.getElementById(ob)) { "
            ."var thisform = document.".$this->get_form_name()."; "
            ."var ovfl = thisform.".$overfill->get_element_name()."; "
            ."if(thisform."._("Range_Type")."[0].checked) {"
            ."ovfl.disabled = true; ovfl.value = '1.00'; "
            ."} else { "
            ."ovfl.disabled = false; "
            ."}} else { "
            ."setTimeout(\"disable('\" + ob + \"')\", 5); "
            ."} "
            ."}";

        $type->set_OnClick($js);
        
        $this->add_elements($name,$start,$end,$enable,$type,$any,$overfill,$comment);

    }

//}}}
//{{{   form_init_data()
        function form_init_data()
        {
            $this->set_element_value(_("Comment"), _("IP Range Created!"));
            $this->set_element_value(_("Name"), _("IP Range"));

            //default multiplier = 1.00
            //This means there are exactly as many free adresses as there are 
            //free ip's.
            $this->set_element_value(_("Overfill Multiplier"), "1.00");
        }
//}}}
//{{{   form_backend_validation()

    /**
     *  This method validates the form values before the are added to the database
     *
     *  @return Boolean     True if data is valid
     */
    function form_backend_validation()
    {
        $res = true; //return value assumes true
        $vals = $this->get_all_element_values(); //get the form vals

        $range = new Range($_REQUEST['id']); //Make a fresh range
        $range->setAll(); //populate with defaults
        $range->setFrom($this->remap($vals, DBDO)); //fill in with form vals

        //If the range is static, then the overfill_muiltiplier will be 1.00 no
        //matter what. This is a nessisary check here because if you edit
        //a dynamic range, and change it to static, the overfill multiplier
        //doesn't get changed, and a static range is then magically overfilled.
        //this check fixes that problem.
        if($range->type == 'static') {
            $range->overfill = '1.00';
        }
        $valid = $range->validate(); //attempt to validate

        if($valid !== true) { //if it wasn't valid do error reporting
            $errors = $this->remap($valid, FORM); //remap the errors to the form elements
            foreach($errors as $label=>$error) { //loop through the errors
                if($error !== true) { //if particular element isn't valid
                    $this->add_error($label, $error);  //report it
                    $res = false;
                }
            }
        }

        return $res; //return our results
    }

//}}}
//{{{   javascript()
    function javascript()
    {
        return $this->java;
    }

//}}}
//{{{   form_action()

    /**
     *  This method performs actions on the data.  In this case inserts it into
     *  the database to create a new Range object
     *
     *  @return Boolean     True if action is successful
     */
    function form_action()
    {
        global $user;
        $res = true;
        
        switch(strtolower($this->get_action())) {

        case 'submit':
            $vals = $this->get_all_element_values();
            $range = new Range();
            $range->query("BEGIN");
            $range->setAll();
            $range->setFrom($this->remap($vals, DBDO));
            $range->setSubnet(null); //Needs to figure out its subnet
            $id = $range->insert();
            $_SESSION['create_id'] = $id;
            if(!$id) { //if insertion fails
                $range->query("ROLLBACK");
                $res = false;
                $this->add_error(_("ERROR"),_("Range couldn't be added to database"));
            } else { //else it worked, so write history
                $range->query("COMMIT");
                $range->write_history($vals[_('Comment')],ACTION_CREATE, 
                                        $user->username, $_SESSION['zone']);
            }
            
            break;
            
        default:
            die_hard(_("Invalid form action"));
            break;

        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This method displays a success message to the user if the action performed
     *  was successful.
     *
     *  @return DIVTag  $div    Div that contains the success message
     */
    function form_success()
    {
        $div = html_div("msuccess");
        $indlink = html_a(web_root()."index.php", _("[Return to Index]"));
        $rnglink = html_a(web_root()."range.php", _("[Return to Range List]"));
        $edit = html_a(web_root()."range.php?action=edit&amp;id=".
                                $_SESSION['create_id'],_("[Edit New Range]"));
        $div->add(_("Range successfully created!"), html_br());
        $div->add($indlink, $rnglink,$edit);

        return $div;
    }

//}}}

}

?>
