<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2006 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id$

require_once INCLUDEDIR."/auth.php";
require_once MODDIR."/auth_maintain/auth_maintain.php";

/**
  * The legacy Maintain user authentication module
  * Handles old MySQL password hashes as opposed to the new SHA1 ones.
  *
  * @package    modules
  * @subpackage maintain_auth
  */
class Auth_Legacy extends Auth_Maintain
{

/* Module Overloads */
// {{{ constructor

    function Auth_Legacy()
    {
        $this->_path = dirname(__FILE__);
        $this->_container = new MaintAuth_Container_Legacy();
    }

// }}}

// abstract installation methods, overridden to allow installation and uninstallation
// {{{ _install()

    /**
      * Perform any actions that need doing before this module
      * is usable.
      *
      * Do not modify the modules table, as this will be done
      * by the {@link Module} class in it's {@link install} method
      *
      * @access protected
      * @return mixed   true on success or PEAR_Error on failure
      */
    function _install() {
        // replace auth_maintain or mysql login methods for existing users to
        // point to this module
        $authusers = DB_DataObject::Factory('user');
        $result = $authusers->query("UPDATE user SET type = 'auth_legacy' WHERE (type = 'auth_maintain' OR type = 'mysql') AND (LENGTH(password) = 16 OR LENGTH(password) = 41);");
        
        return (!DB::isError($result)); // return true if no error
    }

// }}}
// {{{ _uninstall()

    /**
      * Perform any actions that need doing to remove this module from Maintain
      * Typically, this involves removing relevant database tables
      *
      * Do not modify the modules table, however, this will be done
      * by the {@link Module} class in it's {@link uninstall} method
      *
      * @access protected
      * @return mixed  true on success, PEAR_Error on failure
      */
    function _uninstall() 
    {
        // make all auth_legacy login methods for existing users point to
        // auth_maintain
        $authusers = DB_DataObject::Factory('user');
        $result = $authusers->query("UPDATE user SET type = 'auth_maintain' WHERE type = 'auth_legacy';");
        
        return (!DB::isError($result)); // return true if no error
    }

// }}}

} //Auth_Maintain class


/**
  * Overloaded methods from the Database container in PEAR_Auth
  * allowing us to do certain tricks like determining the password
  * hash to use based on MySQL version, etc.
  *
  * @package    classes
  * @subpackage auth
  */
class MaintAuth_Container_Legacy extends MaintAuth_Container_DB {

//{{{ Constructor

/**
 * Empty constructor.
 * Hits the MaintAuth constructor which will create a working
 * PEAR_Auth container environment
 */
function MaintAuth_Container_Legacy() 
{
    parent::MaintAuth_Container_DB();
}

//}}}
    // {{{ fetchData()

    /**
      * Overridden from Auth_Container_DB.
      *
      * This is being loaded so that we can
      * update the user's password hash if needed
      * but the internal value of $username isn't set
      * until fetchData has completed successfully
      */
    function fetchData($username, $password) {
        $result = parent::fetchData($username, $password);

        // see if we're using 'password' as a cryptType
        $crypt = strtolower($this->options['cryptType']);
        // if the login succeeded and we're using 'password' to crypt,
        // then convert the password to the newest hash
        if (!Pear::isError($result) && $result === true && !strcmp($crypt,'password')) {
            $result = $this->update_hash($username, $password);
            if (!$result)
                die("was unable to hash your password.");
        }

        return $result;
    }

    // }}}
    // {{{ verifyPassword()

    /**
     * Crypt and verify the entered password
     * 
     * @access public
     * @global mixed $conf Maintain global config array
     * @param  string Entered password
     * @param  string Password from the data container (usually this password
     *                is already encrypted).
     * @param  string Type of algorithm with which the password from
     *                the container has been crypted. (md5, crypt etc.)
     *                Defaults to "md5".
     * @return bool   True, if the passwords match
     */
    function verifyPassword($password1, $password2, $cryptType)
    {
        global $conf;

        // since we have to specify the password encryption _before_ we know who the user
        // is and how their password is hashed, we have the luxury of updating the cryptType
        // here based on the length of the hash
        switch (strlen($password2))
        {
            case MYSQL_OLD_PASSWORD_LEN:
                $cryptType = 'password';
                break;

            default:
                $cryptType = strtolower($cryptType);
        }

        // update the object's cryptType accordingly
        $this->options['cryptType'] = $cryptType;

        // default return value
        $value = false;

        switch($cryptType) {

            case "password":

                // take the user's password, and MySQL<4.1 PASSWORD()-encrypt it
                // note: for v3.2.3 <= MySQL version < 4.0.11  use PASSWORD(),
                //  otherwise, use OLD_PASSWORD()
                // connect to the DB
                $connect = mysql_connect($conf['database']['host'], $conf['database']['username'], $conf['database']['password']);
                
                // choose the proper database
                mysql_select_db($conf['database']['database']);
                /* Here's how we have to password hash:
                 *
                 * Get the mysql version string from the DB
                 * Extract just the vers[.vers]* part
                 * compare each part of the version
                 */

                // get the current MySQL version
                $res = mysql_query("SELECT version()");
                $row = mysql_fetch_row($res);
                $row = $row[0];
                
                // grab the version digits, i.e. 4.0.11
                ereg("^[([:digit:]+).]+", $row, $vers);

                // split it to an array of each part
                $vers = explode(".", $vers[0]);

                $hash = '';
                switch ($vers[0]) {

                    // version 3.x uses the old password hash
                    case "3":
                        $hash = 'PASSWORD';
                        break;

                    // versions < 4.0.11 use old passsword hash
                    // higher versions need to use OLD_PASSWORD
                    case "4":
                        if (!isset($vers[1]))
                            $hash = 'PASSWORD';

                        elseif ($vers[1] > "0")
                            $hash = 'OLD_PASSWORD';

                        // if it's "0", we need to keep checking
                        else {
                            if (!isset($vers[2]) || $vers[2] < "11" )
                                $hash = 'PASSWORD';
                            else
                                $hash = 'OLD_PASSWORD';
                        }
                        break;
                    default:
                        return PEAR::raiseError("Could not determine MySQL version", PEAR_ERROR_DIE);

                } //switch major version

                // finally, we can get the hashed password
                $res = mysql_query("SELECT $hash('$password1')", dbh());
               
                // make sure we got it
                if (!is_resource($res))
                    return PEAR::raiseError("Could not query server for MySQL PASSWORD() hash", PEAR_ERROR_DIE);

                $row = mysql_fetch_row($res);

                if (!is_array($row))
                    return PEAR::raiseError("Server returned no MySQL PASSWORD() hash", PEAR_ERROR_DIE);

                $value = (strcmp($row[0], $password2)) ? false : true;

                // end of 'password' case
                break;

            default:
                $value = parent::verifyPassword($password1,$password2, $cryptType);
    
        } // switch cryptType
        // return our findings
        return $value;

    } //END verifyPassword()

    //}}}
    // {{{ update_hash()

    /**
      * Convert an password that was hashed using
      * an older hash method to the newest hash algorithm
      * supported by Maintain, also set auth type to
      * auth_maintain rather than auth_legacy
      *
      * This is designed as a helper function for verifyPassword
      *
      * @private
      * @param  string  username of user that needs new password hash
      * @param  string  new, unhashed password
      * @return boolean true on successful change
      */
    function update_hash($username, $newpass) {
        $user = DB_DataObject::Factory('user');
        $user->get($username);
        $user->setPassword(sha1($newpass));
        $user->setType('auth_maintain');
        $result = $user->update();
        
        return (($result) ? true : false);

    }

    // }}}

} // MaintAuth_Container

?>
